﻿using System;

using Prism.Commands;
using Prism.Interactivity.InteractionRequest;
using Prism.Mvvm;

using TestProject.Notifications;

namespace TestProject.ViewModels
{
    /// <summary>
    /// 항목 선택 뷰 모델
    /// </summary>
    public class ItemSelectionViewModel : BindableBase, IInteractionRequestAware
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 통지
        /// </summary>
        private ICustomNotification notification;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 선택 항목 - SelectedItem

        /// <summary>
        /// 선택 항목
        /// </summary>
        public string SelectedItem { get; set; }

        #endregion
        #region 항목 선택 명령 - SelectItemCommand

        /// <summary>
        /// 항목 선택 명령
        /// </summary>
        public DelegateCommand SelectItemCommand { get; private set; }

        #endregion
        #region 취소 명령 - CancelCommand

        /// <summary>
        /// 취소 명령
        /// </summary>
        public DelegateCommand CancelCommand { get; private set; }

        #endregion
        #region 종료 상호 작용 - FinishInteraction

        /// <summary>
        /// 종료 상호 작용
        /// </summary>
        public Action FinishInteraction { get; set; }

        #endregion
        #region 통지 - Notification

        /// <summary>
        /// 통지
        /// </summary>
        public INotification Notification
        {
            get
            {
                return this.notification;
            }
            set
            {
                SetProperty(ref this.notification, (ICustomNotification)value);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ItemSelectionViewModel()

        /// <summary>
        /// 생성자
        /// </summary>
        public ItemSelectionViewModel()
        {
            SelectItemCommand = new DelegateCommand(SelectItem);
            CancelCommand     = new DelegateCommand(Cancel);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 항목 선택하기 - SelectItem()

        /// <summary>
        /// 항목 선택하기
        /// </summary>
        private void SelectItem()
        {
            this.notification.SelectedItem = SelectedItem;
            this.notification.Confirmed    = true;

            FinishInteraction?.Invoke();
        }

        #endregion
        #region 취소하기 - Cancel()

        /// <summary>
        /// 취소하기
        /// </summary>
        private void Cancel()
        {
            this.notification.SelectedItem = null;
            this.notification.Confirmed    = false;

            FinishInteraction?.Invoke();
        }

        #endregion
    }
}