﻿using Prism.Commands;
using Prism.Interactivity.InteractionRequest;
using Prism.Mvvm;

using TestProject.Notifications;

namespace TestProject.ViewModels
{
    /// <summary>
    /// 메인 윈도우 뷰 모델
    /// </summary>
    public class MainWindowViewModel : BindableBase
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 제목
        /// </summary>
        private string title = "InteractionRequest<T> 클래스 : 커스텀 대화 상자 표시하기";

        /// <summary>
        /// 메시지
        /// </summary>
        private string message = "";

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 제목 - Title

        /// <summary>
        /// 제목
        /// </summary>
        public string Title
        {
            get
            {
                return this.title;
            }
            set
            {
                SetProperty(ref this.title, value);
            }
        }

        #endregion
        #region 메시지 - Message

        /// <summary>
        /// 메시지
        /// </summary>
        public string Message
        {
            get
            {
                return this.message;
            }
            set
            {
                SetProperty(ref this.message, value);
            }
        }

        #endregion

        #region 통지 요청 - NotificationRequest

        /// <summary>
        /// 통지 요청
        /// </summary>
        public InteractionRequest<INotification> NotificationRequest { get; set; }

        #endregion
        #region 통지 명령 - NotificationCommand

        /// <summary>
        /// 통지 명령
        /// </summary>
        public DelegateCommand NotificationCommand { get; set; }

        #endregion
        #region 확인 요청 - ConfirmationRequest

        /// <summary>
        /// 확인 요청
        /// </summary>
        public InteractionRequest<IConfirmation> ConfirmationRequest { get; set; }

        #endregion
        #region 확인 명령 - ConfirmationCommand

        /// <summary>
        /// 확인 명령
        /// </summary>
        public DelegateCommand ConfirmationCommand { get; set; }

        #endregion
        #region 커스텀 팝업 요청 - CustomPopupRequest

        /// <summary>
        /// 커스텀 팝업 요청
        /// </summary>
        public InteractionRequest<INotification> CustomPopupRequest { get; set; }

        #endregion
        #region 커스텀 팝업 명령 - CustomPopupCommand

        /// <summary>
        /// 커스텀 팝업 명령
        /// </summary>
        public DelegateCommand CustomPopupCommand { get; set; }

        #endregion
        #region 커스텀 통지 요청 - CustomNotificationRequest

        /// <summary>
        /// 커스텀 통지 요청
        /// </summary>
        public InteractionRequest<ICustomNotification> CustomNotificationRequest { get; set; }

        #endregion
        #region 커스텀 통지 명령 - CustomNotificationCommand

        /// <summary>
        /// 커스텀 통지 명령
        /// </summary>
        public DelegateCommand CustomNotificationCommand { get; set; }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindowViewModel()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindowViewModel()
        {
            NotificationRequest = new InteractionRequest<INotification>();
            NotificationCommand = new DelegateCommand(Notify);

            ConfirmationRequest = new InteractionRequest<IConfirmation>();
            ConfirmationCommand = new DelegateCommand(Confirm);

            CustomPopupRequest = new InteractionRequest<INotification>();
            CustomPopupCommand = new DelegateCommand(ShowCusomPopup);

            CustomNotificationRequest = new InteractionRequest<ICustomNotification>();
            CustomNotificationCommand = new DelegateCommand(ShowCustomRequest);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 통지하기 - Notify()

        /// <summary>
        /// 통지하기
        /// </summary>
        private void Notify()
        {
            NotificationRequest.Raise
            (
                new Notification
                {
                    Content = "Notification Message",
                    Title   = "Notification"
                },
                r => Message = "Notified"
            );
        }

        #endregion
        #region 확인하기 - Confirm()

        /// <summary>
        /// 확인하기
        /// </summary>
        private void Confirm()
        {
            ConfirmationRequest.Raise
            (
                new Confirmation
                {
                    Title   = "Confirmation",
                    Content = "Confirmation Message"
                },
                r => Message = r.Confirmed ? "Confirmed" : "Not Confirmed"
            );
        }

        #endregion
        #region 커스텀 팝업 표시하기 - ShowCusomPopup()

        /// <summary>
        /// 커스텀 팝업 표시하기
        /// </summary>
        private void ShowCusomPopup()
        {
            CustomPopupRequest.Raise
            (
                new Notification
                {
                    Title   = "Custom Popup",
                    Content = "Custom Popup Message"
                },
                r => Message = "Good to go"
            );
        }

        #endregion
        #region 커스텀 요청 표시하기 - ShowCustomRequest()

        /// <summary>
        /// 커스텀 요청 표시하기
        /// </summary>
        private void ShowCustomRequest()
        {
            CustomNotificationRequest.Raise
            (
                new CustomNotification
                {
                    Title = "Custom Notification"
                },
                r =>
                {
                    if(r.Confirmed && r.SelectedItem != null)
                    {
                        Message = $"User selected : { r.SelectedItem}";
                    }
                    else
                    {
                        Message = "User cancelled or didn't select an item";
                    }
                }
            );
        }

        #endregion
    }
}